package com.isoft91.gonn.gps.business.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.isoft91.gonn.gps.business.dto.QuerySDeptDTO;
import com.isoft91.gonn.gps.business.dto.ResultSDeptDTO;
import com.isoft91.gonn.gps.business.entity.SDept;
import com.isoft91.gonn.gps.business.service.ISDeptService;
import com.isoft91.gonn.gps.utils.DozerUtils;
import com.isoft91.gonn.gps.utils.Result.Ret;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 部门表 前端控制器
 * </p>
 *
 * @author 91isoft
 * @since 2019-12-30
 */
@RestController
@RequestMapping("/sdept")
public class SDeptController {
    
    @Autowired
    ISDeptService isDeptServiceImpl;

    @Transactional(rollbackFor = Exception.class)
    @PutMapping("/{businessId:\\w+}")
    @ApiOperation(value = "修改部门信息", notes = "")
    public Ret updateDeptById(@PathVariable(value = "businessId") String businessId, QuerySDeptDTO sDeptDTO) {
        SDept sDept = DozerUtils.map(sDeptDTO, SDept.class);
        //判断id 是否为空
        if (StrUtil.isNotEmpty(businessId)) {
            sDept.setBusinessId(businessId);
            if (isDeptServiceImpl.updateById(sDept)) {
                return Ret.ok().setMsg("修改成功").setData(sDept);
            } else {
                return Ret.error().setMsg("修改失败");
            }
        } else {
            return Ret.error().setMsg("deptId不能为空");
        }
    }


    @Transactional(rollbackFor = Exception.class)
    @DeleteMapping("/{businessId:\\w+}")
    @ApiOperation(value = "物理删除部门信息", notes = "")
    public Ret removeDeptById(@PathVariable(value = "businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId)) {
            return Ret.ok().setData(isDeptServiceImpl.delete(businessId));
        } else {
            return Ret.error().setMsg("deptId不能为空");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @PutMapping("/logDel/{businessId:\\w+}")
    @ApiOperation(value = "逻辑删除部门", notes = "")
    public Ret logicalDeleteDeptById(@PathVariable(value = "businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId) && isDeptServiceImpl.logicalDelete(businessId) > 0) {
            return Ret.ok().setMsg("操作成功");
        } else {
            return Ret.error().setMsg("deptId不能为空");
        }
    }


    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    @GetMapping
    @ApiOperation(value = "查询所有部门信息(不带分页)", notes = "查询所有部门信息列表(传id即查询编辑页信息)")
    public Ret<List<ResultSDeptDTO>> selectDeptList(QuerySDeptDTO querySDeptDTO) {

        return Ret.ok().setData(isDeptServiceImpl.getDeptList(querySDeptDTO));
    }

    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    @GetMapping("/page")
    @ApiOperation(value = "分页部门信息", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页", required = true),
            @ApiImplicitParam(name = "size", value = "页大小", required = true)})
    public Ret<IPage<ResultSDeptDTO>> selectDeptListPage(QuerySDeptDTO querySDeptDTO, Integer page, Integer size) {
        if (page == null && size == null) {
            return Ret.error().setMsg("当前页和页大小不能为空");
        }
        return Ret.ok().setData(isDeptServiceImpl.getDeptListPage(new Page<>(page, size), querySDeptDTO));
    }


}
