package com.isoft91.gonn.gps.business.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.isoft91.gonn.gps.business.dto.*;
import com.isoft91.gonn.gps.business.entity.SDictionary;
import com.isoft91.gonn.gps.business.service.ISDictionaryService;
import com.isoft91.gonn.gps.utils.DozerUtils;
import com.isoft91.gonn.gps.utils.Result.Ret;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 数据字典 前端控制器
 * </p>
 *
 * @author 91isoft
 * @since 2019-12-25
 */
@RestController
@RequestMapping("/sdictionary")
public class SDictionaryController {

    @Autowired
    ISDictionaryService dictionaryServiceImpl;

    @Transactional(rollbackFor=Exception.class)
    @PostMapping
    @ApiOperation(value = "添加数据字典", notes = "添加数据字典")
    @ApiImplicitParams({
            @ApiImplicitParam(name  =  "type" ,value  ="数据字典type",required  =  false),
            @ApiImplicitParam(name  =  "name",value  ="数据字典名称"),
            @ApiImplicitParam(name  =  "value",value  ="数据字典value")
    })
    public Ret insert(QuerySDictionaryDTO dictionaryDto) {
        SDictionary dictionary = DozerUtils.map(dictionaryDto, SDictionary.class);
        if (dictionaryServiceImpl.save(dictionary) ) {
            return Ret.ok().setMsg("添加成功").setData(dictionaryDto);
        } else {
            return Ret.error().setMsg("添加失败");
        }

    }

    @Transactional(rollbackFor=Exception.class)
    @PutMapping("/{businessId:\\w+}")
    @ApiOperation(value = "修改数据字典", notes = "")
    public Ret updateById(@PathVariable(value = "businessId") String businessId, QuerySDictionaryDTO dictionaryDto) {
        SDictionary dictionary = DozerUtils.map(dictionaryDto, SDictionary.class);
        //判断id 是否为空
        if (StrUtil.isNotEmpty(businessId)) {
            dictionary.setBusinessId(businessId);
            if (dictionaryServiceImpl.updateById(dictionary)) {
                return Ret.ok().setMsg("修改成功").setData(dictionary);
            } else {
                return Ret.error().setMsg("修改失败");
            }
        } else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }



    @Transactional(rollbackFor=Exception.class)
    @DeleteMapping("/{businessId:\\w+}")
    @ApiOperation(value = "物理删除数据字典", notes = "")
    public Ret removeById(@PathVariable(value="businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId)) {
            return Ret.ok().setData(dictionaryServiceImpl.delete(businessId));
        }else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }

    @Transactional(rollbackFor=Exception.class)
    @PutMapping("/logDel/{businessId:\\w+}")
    @ApiOperation(value = "逻辑删除数据字典", notes = "")
    public Ret logicalDeleteById(@PathVariable(value="businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId) && dictionaryServiceImpl.logicalDelete(businessId) > 0) {
            return Ret.ok().setMsg("操作成功");
        }else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("byId")
    @ApiOperation(value = "根据id查询数据字典列表")
    public Ret<ResultSDictionaryDTO> selectDicById(String businessId) {

        return Ret.ok().setData(dictionaryServiceImpl.getDictionaryByID(businessId));
    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping
    @ApiOperation(value = "查询所有数据字典列表(不带分页)", notes = "查询数据字典列表(传id即查询编辑页信息)")
    public Ret<List<ResultSDictionaryDTO>> selectList(QuerySDictionaryDTO dictionary) {

        return Ret.ok().setData(dictionaryServiceImpl.getDictionaryList(dictionary));
    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/page")
    @ApiOperation(value = "分页查询数据字典", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页", required = true),
            @ApiImplicitParam(name = "size", value = "页大小", required = true)})
    public Ret<IPage<ResultSDictionaryDTO>> selectListPage(QuerySDictionaryDTO sDictionary, Integer page, Integer size) {
        if (page == null && size == null) {
            return Ret.error().setMsg("当前页和页大小不能为空");
        }
        return Ret.ok().setData(dictionaryServiceImpl.getDictionaryListPage(new Page<>(page,size),sDictionary));
    }


    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/pageBy")
    @ApiOperation(value = "条件分页查询数据字典")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页", required = true),
            @ApiImplicitParam(name = "size", value = "页大小", required = true)})
    public Ret<IPage> selectListPageBy(QuerySDictionaryDTO querySDictionaryDTO, Integer page, Integer size) {
        if (page == null && size == null) {
            return Ret.error().setMsg("当前页和页大小不能为空");
        }
        IPage iPage = dictionaryServiceImpl.getDictionaryListPageBy(new Page<>(page, size),querySDictionaryDTO);
        return Ret.ok().setData(iPage);

    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED)
    @GetMapping("/getDictionaryListByType")
    @ApiOperation(value = "根据类型查询数据字典")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "数据字典类型", required = true)})
    public Ret<IPage<ResultSDictionaryDTO>> getDictionaryListByType(String type) {
        if (type == null || type == ""){
            return Ret.error().setMsg("数据的字典类型不能为空");
        }
        List dictionaryListByType = dictionaryServiceImpl.getDictionaryListByType(type);
        return Ret.ok().setData(dictionaryListByType);

    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/getGradeDic")
    @ApiOperation(value = "获取业务级联数据(0310)")
    @ApiImplicitParams({
//            @ApiImplicitParam(name = "type", value = "数据字典类型", required = false)
            })
    public Ret<IPage<ResultSDictionaryDTO>> getGradeDic() {
        Map dictionaryListByType = dictionaryServiceImpl.getGradeDic();
        return Ret.ok().setData(dictionaryListByType);

    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/getTemplateType")
    @ApiOperation(value = "查询所有模板类型")
    public Ret<List<ResultSearchDTO>> selectTemplateTypeList() {

        return Ret.ok().setData(dictionaryServiceImpl.getTemplateTypeList());
    }

    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/getTemplateGrade")
    @ApiOperation(value = "查询所有模板级别")
    public Ret<List<ResultSearchDTO>> selectTemplateGradeList() {

        return Ret.ok().setData(dictionaryServiceImpl.getTemplateGradeList());
    }
    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/getProjecetStage")
    @ApiOperation(value = "查询项目阶段")
    public Ret<List<SdProjectStageDTO>> getProjecetStage(){
        return dictionaryServiceImpl.getProjecetStage();
    }


    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/getServiceStage")
    @ApiOperation(value = "查询业务阶段")
    public Ret<List<SdServiceStageDTO>> getServiceStage(){
        return dictionaryServiceImpl.getServiceStage();
    }
}

