package com.isoft91.gonn.gps.business.controller;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.isoft91.gonn.gps.business.dto.QuerySRoleDTO;
import com.isoft91.gonn.gps.business.dto.ResultSDictionaryDTO;
import com.isoft91.gonn.gps.business.dto.ResultSRoleDTO;
import com.isoft91.gonn.gps.business.entity.SRole;
import com.isoft91.gonn.gps.business.service.ISRoleService;
import com.isoft91.gonn.gps.utils.DozerUtils;
import com.isoft91.gonn.gps.utils.Result.Ret;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 角色表 前端控制器
 * </p>
 *
 * @author 91isoft
 * @since 2019-12-30
 */
@RestController
@RequestMapping("/srole")
public class SRoleController {

    @Autowired
    ISRoleService isRoleServiceImpl;


    @Transactional(rollbackFor=Exception.class)
    @PutMapping("/{businessId:\\w+}")
    @ApiOperation(value = "修改角色信息", notes = "")
    public Ret updateRoleById(@PathVariable(value="businessId") String businessId, QuerySRoleDTO sRoleDTO) {
        SRole sRole = DozerUtils.map(sRoleDTO, SRole.class);
        //判断id 是否为空
        if (StrUtil.isNotEmpty(businessId)) {
            sRole.setBusinessId(businessId);
            if (isRoleServiceImpl.updateById(sRole)) {
                return Ret.ok().setMsg("修改成功").setData(sRole);
            } else {
                return Ret.error().setMsg("修改失败");
            }
        } else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }

    @Transactional(rollbackFor=Exception.class)
    @DeleteMapping("/{businessId:\\w+}")
    @ApiOperation(value = "物理删除角色信息", notes = "")
    public Ret removeRoleById(@PathVariable(value="businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId)) {
            return Ret.ok().setData(isRoleServiceImpl.delete(businessId));
        }else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }

    @Transactional(rollbackFor=Exception.class)
    @PutMapping("/logDel/{businessId:\\w+}")
    @ApiOperation(value = "逻辑删除角色信息", notes = "")
    public Ret logicalDeleteRoleById(@PathVariable(value="businessId") String businessId) {
        if (StrUtil.isNotBlank(businessId) && isRoleServiceImpl.logicalDelete(businessId) > 0) {
            return Ret.ok().setMsg("操作成功");
        }else {
            return Ret.error().setMsg("businessId不能为空");
        }
    }

    @Transactional(propagation= Propagation.NOT_SUPPORTED )
    @GetMapping
    @ApiOperation(value = "查询所有角色信息列表(不带分页)", notes = "查询所有角色信息列表(传id即查询编辑页信息)")
    public Ret<List<ResultSRoleDTO>> selectRoleList(QuerySRoleDTO querySRoleDTO) {

        return Ret.ok().setData(isRoleServiceImpl.getRoleList(querySRoleDTO));
    }


    @Transactional(propagation=Propagation.NOT_SUPPORTED )
    @GetMapping("/page")
    @ApiOperation(value = "分页查询角色信息", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页", required = true),
            @ApiImplicitParam(name = "size", value = "页大小", required = true)})
    public Ret<IPage<ResultSDictionaryDTO>> selectListPage(QuerySRoleDTO querySRoleDTO, Integer page, Integer size) {
        if (page == null && size == null) {
            return Ret.error().setMsg("当前页和页大小不能为空");
        }
        return Ret.ok().setData(isRoleServiceImpl.getRoleListPage(new Page<>(page,size),querySRoleDTO));
    }
}
