package com.isoft91.gonn.gps.business.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isoft91.gonn.gps.business.dto.*;
import com.isoft91.gonn.gps.business.entity.SDictionary;
import com.isoft91.gonn.gps.business.mapper.SDictionaryMapper;
import com.isoft91.gonn.gps.business.service.ISDictionaryService;
import com.isoft91.gonn.gps.common.entity.DelStatus;
import com.isoft91.gonn.gps.common.entity.FlagStatus;
import com.isoft91.gonn.gps.utils.DozerUtils;
import com.isoft91.gonn.gps.utils.Result.Ret;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 数据字典 服务实现类
 * </p>
 *
 * @author 91isoft
 * @since 2019-12-25
 */
@Service
public class SDictionaryServiceImpl extends ServiceImpl<SDictionaryMapper, SDictionary> implements ISDictionaryService {

    @Value("${dictionary.mapper}")
    private String mapper;

    @Value("${dictionary.templateType.service}")
    private String serviceType;

    @Value("${dictionary.templateType.project}")
    private String projectType;

    @Value("${dictionary.templateStage.project}")
    private String projectStageType;

    @Value("${dictionary.templateStage.business}")
    private String businessStageType;


    @Value("${dictionary.templateStage.service}")
    private String serviceStageType;

    @Value("${dictionary.tempType}")
    private String tempType;


    @Autowired
    SDictionaryMapper sDictionaryMapper;


    @Override
    public int logicalDelete(String id) {
        SDictionary sDictionary = new SDictionary();
        sDictionary.setDeleted();
        sDictionary.setStop();
        sDictionary.setBusinessId(id);
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("business_id", id);
        int line = this.baseMapper.updateById(sDictionary);
        return line;
    }

    @Override
    public int delete(String id) {
        return this.baseMapper.deleteById(id);
    }

    @Override
    public List<ResultSDictionaryDTO> getDictionaryList(QuerySDictionaryDTO sDictionary) {
        QueryWrapper queryWrapper = getQueryWrapper(sDictionary);
        List<SDictionary> sDictionaries = this.baseMapper.selectList(queryWrapper);
        List<ResultSDictionaryDTO> resultSictionaryDtos = DozerUtils.mapList(sDictionaries, ResultSDictionaryDTO.class);
        return resultSictionaryDtos;
    }

    @Override
    public SDictionary getDictionaryByID(String sDictionaryId) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        queryWrapper.eq("BUSINESS_ID", sDictionaryId);
        return this.baseMapper.selectOne(queryWrapper);
    }

    @Override
    public IPage getDictionaryListPage(Page page, QuerySDictionaryDTO sDictionary) {
        QueryWrapper queryWrapper = getQueryWrapper(sDictionary);
        return this.baseMapper.selectPage(page, queryWrapper);
    }


    /**
     * 条件分页查询数据字典
     */
    @Override
    public IPage getDictionaryListPageBy(Page page, QuerySDictionaryDTO querySDictionaryDTO) {
        /*QueryWrapper<SDictionary> queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        queryWrapper.orderByDesc("create_date");
        queryWrapper.like(StringUtils.isNotBlank(querySDictionaryDTO.getName()), "name", querySDictionaryDTO.getName())
                .like(StringUtils.isNotBlank(querySDictionaryDTO.getType()), "type", querySDictionaryDTO.getType())
                .like(StringUtils.isNotBlank(querySDictionaryDTO.getValue()), "value", querySDictionaryDTO.getValue())
                .eq(StringUtils.isNotBlank(querySDictionaryDTO.getPid()), "PID", querySDictionaryDTO.getPid())
                .eq(StringUtils.isNotBlank(querySDictionaryDTO.getBusinessId()), "BUSINESS_ID", querySDictionaryDTO.getBusinessId());*/
        querySDictionaryDTO.setFlag(FlagStatus.NORMAL.getStatus());
        querySDictionaryDTO.setDelFlag(DelStatus.NORMAL.getStatus());
        return this.baseMapper.selSDictionaryPage(querySDictionaryDTO,page);
    }

    /**
     * 根据类型查询数据字典
     *
     * @param type
     * @return
     */
    @Override
    public List getDictionaryListByType(String type) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        queryWrapper.eq("TYPE", type);
        return this.baseMapper.selectList(queryWrapper);
    }

    @Override
    public Map getGradeDic() {
        Map map = new HashMap();
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("TYPE", projectType);
        List<SDictionary> dictionaries =  this.baseMapper.selectList(wrapper);
        List<ResultSDictionaryDTO> resultSDictionaryDTOS = DozerUtils.mapList(dictionaries,ResultSDictionaryDTO.class);
        map.put("projectStage",resultSDictionaryDTOS);
        wrapper = new QueryWrapper();
        wrapper.eq("TYPE",businessStageType);
        dictionaries =  this.baseMapper.selectList(wrapper);
        resultSDictionaryDTOS = DozerUtils.mapList(dictionaries,ResultSDictionaryDTO.class);
        map.put("businessStage",resultSDictionaryDTOS);
        wrapper = new QueryWrapper();
        wrapper.eq("TYPE",serviceStageType);
        map.put("serviceStage",sDictionaryMapper.selectGradeList(serviceType,mapper));
        return map;
    }

    @Override
    public List<ResultSearchDTO> getTemplateTypeList() {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        queryWrapper.eq("type", tempType);
        List<SDictionary> sDictionaries = this.baseMapper.selectList(queryWrapper);
        List<ResultSearchDTO> resultTemplateDTOs = DozerUtils.mapList(sDictionaries, ResultSearchDTO.class);
        for (ResultSearchDTO dto: resultTemplateDTOs){
            if (serviceType.equals(dto.getValue())){
                queryWrapper = new QueryWrapper();
                queryWrapper.eq("type",serviceType);
                dto.setChild(DozerUtils.mapList(sDictionaryMapper.selectList(queryWrapper),ResultSDictionaryDTO.class));
            }
        }
        return resultTemplateDTOs;
    }

    @Override
    public List<ResultSearchDTO> getTemplateGradeList() {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        queryWrapper.eq("type", "TEMP_GRADE");
        List<SDictionary> sDictionaries = this.baseMapper.selectList(queryWrapper);
        List<ResultSearchDTO> resultTemplateDTOs = DozerUtils.mapList(sDictionaries, ResultSearchDTO.class);
        return resultTemplateDTOs;
    }

    /**
     * 查询项目数据字典项目阶段
     * []
     *
     * @return {@link Ret<List<SdProjectStageDTO>>}
     * @throws
     * @author jiangpengpeng
     * @date 2020/2/19 14:03
     */
    @Override
    public Ret<List<SdProjectStageDTO>> getProjecetStage() {
        SDictionary sDictionary = new SDictionary();
        sDictionary.setType(projectStageType);
        sDictionary.setNormal();
        List<SdProjectStageDTO> projecetStage = sDictionaryMapper.getProjecetStage(sDictionary);
        if (projecetStage.size() == 0) {
            return Ret.error().setMsg("未查找到项目阶段");
        }
        return Ret.ok().setData(projecetStage);

    }

    @Override
    public Ret<List<SdServiceStageDTO>> getServiceStage() {
        SDictionary sDictionary = new SDictionary();
        sDictionary.setType(serviceStageType);
        sDictionary.setNormal();
        List<SdServiceStageDTO> serviceStageDTOList = sDictionaryMapper.getServiceStage(sDictionary);
        if (serviceStageDTOList.size() == 0) {
            return Ret.error().setMsg("未查找到项目阶段");
        }
        return Ret.ok().setData(serviceStageDTOList);

    }


    private QueryWrapper getQueryWrapper(QuerySDictionaryDTO sDictionary) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.ne("del_flag", "1");
        queryWrapper.ne("flag", "0");
        if (StrUtil.isNotBlank(sDictionary.getBusinessId())) {
            queryWrapper.eq("busienss_id", sDictionary.getBusinessId());
        }
        if (StrUtil.isNotBlank(sDictionary.getType())) {
            queryWrapper.eq("type", sDictionary.getType());
        }
        if (StrUtil.isNotBlank(sDictionary.getName())) {
            queryWrapper.eq("name", sDictionary.getName());
        }
        if (StrUtil.isNotBlank(sDictionary.getValue())) {
            queryWrapper.eq("value", sDictionary.getValue());
        }
        return queryWrapper;
    }
}
