package com.isoft91.gonn.gps.common.aop;

import io.swagger.annotations.ApiOperation;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

/**
 * 系统日志：切面处理类
 */
@Aspect
@Component
public class SysLogAspect implements Ordered {

    private static Logger logger ;
    //定义切点 @Pointcut
    @Pointcut("execution(* com.isoft91.gonn.gps.business.*.controller..*.*(..)),order=3") //切点
    public void webRequestLog(){}


    @Pointcut("execution(* com.isoft91.gonn.gps.business.*.controller..*.*(..)),order=3") //切点
    public void webExceptionLog(){}

    @Before("webRequestLog()")
    @Order(1)
    public void doBefore(JoinPoint joinPoint){
        try {


         } catch (Exception e) {
            e.printStackTrace();
         }
    }

    /**
      * 异常通知 用于拦截异常日志
      *
      * @param joinPoint
      * @param e
      */
    @Order(0)
    @AfterThrowing(pointcut = "webExceptionLog()", throwing = "e")
 public  void doAfterThrowing(JoinPoint joinPoint, Throwable e) {
    HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    try {

    } catch (Exception ex) {
          //记录本地异常日志
          e.printStackTrace();
          logger.error("==异常通知异常==");
      }
    }

    @Override
    public int getOrder() {
        return 0;
    }

    /**
     * 获取 方法上的注释
     * @param joinPoint
     * @return
     */
    private String getMethodMess(JoinPoint joinPoint){
        String value = "";
        try {
            Signature signature = joinPoint.getSignature();
            MethodSignature methodSignature = (MethodSignature) signature;
            Method targetMethod = methodSignature.getMethod();

            Method realMethod = joinPoint.getTarget().getClass().getDeclaredMethod(signature.getName(), targetMethod.getParameterTypes());
            if (realMethod.isAnnotationPresent(ApiOperation.class)) {
                ApiOperation apiOperation = (ApiOperation) realMethod.getAnnotation(ApiOperation.class);
                value = apiOperation.value();
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
        return value;
    }
}
