package com.isoft91.gonn.gps.config.filter;

import com.alibaba.fastjson.JSON;
import com.isoft91.gonn.gps.config.exception.UsernameNotExistException;
import com.isoft91.gonn.gps.sys.entity.User;
import com.isoft91.gonn.gps.sys.service.UserDetailsServiceImpl;
import com.isoft91.gonn.gps.utils.Result.ResultEnums;
import com.isoft91.gonn.gps.utils.Result.Ret;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
/**
 * 登陆信息 验证过滤器: JwtLoginFilter
 */
public class JwtLoginFilter extends AbstractAuthenticationProcessingFilter {
    @Autowired
    private UserDetailsServiceImpl userDetailsService;

    @Value("${jwt.tokenHeader}")
    private String tokenHeader;

    @Value("${jwt.tokenPrefix}")
    private String tokenPrefix;

    @Value("${jwt.secret}")
    private String secret;

    @Value("${jwt.expiration}")
    private int expiration;

    public JwtLoginFilter(String defaultFilterProcessesUrl, AuthenticationManager authenticationManager) {
        super(new AntPathRequestMatcher(defaultFilterProcessesUrl));
        setAuthenticationManager(authenticationManager);
    }

    /**
     * 接收并解析用户登陆信息  /login,为已验证的用户返回一个已填充的身份验证令牌，表示成功的身份验证
     */
    @Override
    public Authentication attemptAuthentication(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws AuthenticationException, IOException, ServletException {
        //得到用户登陆信息,并封装到 Authentication 中,供自定义用户组件使用.
        //这里并没有对用户名密码进行验证,而是使用 AuthenticationProvider 提供的 authenticate 方法
        //返回一个完全经过身份验证的对象
        return getAuthenticationManager().authenticate(new UsernamePasswordAuthenticationToken( (String)httpServletRequest.getParameter("username"),  (String)httpServletRequest.getParameter("password")));
    }


    /**
     *
     * 登陆成功后,此方法会被调用,因此我们可以在次方法中生成token,并返回给客户端
     */
    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain, Authentication authResult) throws IOException, ServletException {
        Collection<? extends GrantedAuthority> authorities = authResult.getAuthorities();

        StringBuffer as = new StringBuffer();
        StringBuilder roleName = new StringBuilder();
        /*  for(GrantedAuthority authority: authorities) {
            as.append(authority.getAuthority()).append(",");
        }*/
        as.append(authResult.getName()).append(",");


//        as.append(userRoles);
        String jwt = Jwts.builder()
                .claim("authorities", as)//配置用户角色
                .setSubject(authResult.getName())
                .setExpiration(new Date(System.currentTimeMillis() + expiration))
                .signWith(SignatureAlgorithm.HS512,secret)
                .compact();
        response.setContentType("application/json;charset=utf-8");
//        response.setHeader("token", jwt);
        PrintWriter out = response.getWriter();
//        out.write(new ObjectMapper().writeValueAsString(jwt));
        Map<String,String> resultMap = new HashMap<>();
        resultMap.put(tokenHeader, jwt);
        resultMap.put("role", roleName.toString());
        User user = (User) authResult.getPrincipal();
        resultMap.put("email", user.getEmail());
        String resJson = JSON.toJSONString(Ret.build(ResultEnums.LOGIN_SUCCESS).setData(resultMap));
        out.write(resJson);
        out.flush();
        out.close();
    }

    @Override
    protected void unsuccessfulAuthentication(HttpServletRequest req, HttpServletResponse resp, AuthenticationException failed) throws IOException, ServletException {
        resp.setContentType("application/json;charset=utf-8");
        PrintWriter out = resp.getWriter();
        String tipInfo;
        if (failed.getCause() instanceof UsernameNotExistException) {
            tipInfo = failed.getMessage();
//        } else if (failed.getCause() instanceof RedisConnectionFailureException) {
//            tipInfo = "Redis连接失败";
        } else if (failed instanceof BadCredentialsException) {
            tipInfo = "用户名或密码错误";
        } else {
            tipInfo = failed.getMessage();
        }
        String resJson = JSON.toJSONString(Ret.build(ResultEnums.LOGIN_ERROR).setData(tipInfo));
        out.write(resJson);
        out.flush();
        out.close();
    }
}

