package com.isoft91.gonn.gps.config.filter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.util.regex.Pattern;

public class XssFilterWrapper extends HttpServletRequestWrapper {

    HttpServletRequest orgRequest = null;
    public XssFilterWrapper(HttpServletRequest request) {
        super(request);
        orgRequest=request;
    }

    /**
     * 覆盖getParameter方法，将参数名和参数值都做xss & sql过滤。<br/>
     * 如果需要获得原始的值，则通过super.getParameterValues(name)来获取<br/>
     * getParameterNames,getParameterValues和getParameterMap也可能需要覆盖
     */
    @Override
    public String getParameter(String name) {
        String value=super.getParameter(xssEncode(name));
        if (value!=null){
            value=xssEncode(value);
        }
        return value;
    }



    @Override
    public String getHeader(String name) {
        String value=super.getHeader(xssEncode(name));
        if (value!=null){
            value=xssEncode(value);
        }
        return value;
    }


/**
 * 将容易引起xss & sql漏洞的半角字符直接替换成全角字符
 *
 * @param s
 * @return
*/
 private  static String xssEncode(String s){
     if(s==null || s.isEmpty()){
         return s;
     }else {
         s=stripXSSAndSql(s);
     }
     StringBuilder sb=new StringBuilder(s.length()+16);
     for (int i=0;i<s.length();i++){
         char c=s.charAt(i);
         switch (c){
             case '>':
                 sb.append("＞");
                 break;
             case '<':
                 sb.append("＜");
                 break;
             case '&':
                 sb.append("＆");// 转义&
                 break;
             case '#':
                 sb.append("＃");// 转义#
                 break;
             default:
                 sb.append(c);
                 break;

         }
     }
    return sb.toString();
 }
    /**
     * 获取最原始的request
     *
     * @return
     */
    public HttpServletRequest getOrgRequest() {
        return orgRequest;
    }
    /**
     * 获取最原始的request的静态方法
     *
     * @return
     */
    public static HttpServletRequest getOrgRequest(HttpServletRequest req) {
        if (req instanceof XssFilterWrapper) {
            return ((XssFilterWrapper) req).getOrgRequest();
        }

        return req;
    }
    public static String stripXSSAndSql(String value){
        if(value!=null){

            Pattern scriptPattern = Pattern.compile("<[\r\n| | ]*script[\r\n| | ]*>(.*?)</[\r\n| | ]*script[\r\n| | ]*>", Pattern.CASE_INSENSITIVE);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid anything in a src="http://www.yihaomen.com/article/java/..." type of e-xpression
            scriptPattern = Pattern.compile("src[\r\n| | ]*=[\r\n| | ]*[\\\"|\\\'](.*?)[\\\"|\\\']", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
            value = scriptPattern.matcher(value).replaceAll("");
            // Remove any lonesome </script> tag
            scriptPattern = Pattern.compile("</[\r\n| | ]*script[\r\n| | ]*>", Pattern.CASE_INSENSITIVE);
            value = scriptPattern.matcher(value).replaceAll("");
            // Remove any lonesome <script ...> tag
            scriptPattern = Pattern.compile("<[\r\n| | ]*script(.*?)>", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid eval(...) expressions
            scriptPattern = Pattern.compile("eval\\((.*?)\\)", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid e-xpression(...) expressions
            scriptPattern = Pattern.compile("e-xpression\\((.*?)\\)", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid javascript:... expressions
            scriptPattern = Pattern.compile("javascript[\r\n| | ]*:[\r\n| | ]*", Pattern.CASE_INSENSITIVE);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid vbscript:... expressions
            scriptPattern = Pattern.compile("vbscript[\r\n| | ]*:[\r\n| | ]*", Pattern.CASE_INSENSITIVE);
            value = scriptPattern.matcher(value).replaceAll("");
            // Avoid οnlοad= expressions
            scriptPattern = Pattern.compile("onload(.*?)=", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
            value = scriptPattern.matcher(value).replaceAll("");
        }
        return value;
    }


}

