package com.isoft91.gonn.gps.sys.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;


/**
 * UserDetails是SpringSecurity验证框架内部提供的用户验证接口，主要是来完成自定义用户认证功能，
 * 需要实现getAuthorities方法内容，将定义的角色列表添加到授权的列表内。
 */
@Data
@TableName("s_user")
public class User implements UserDetails {
    @TableId("BUSINESS_ID")
    /**主键id*/
    private String businessId;
    /**
     * 用户名
     */
    private String loginName;
    /**
     * 密码
     */
    private String password;
    /**
     * 姓名
     */
    private String name;
    /**
     * 性别
     */
    private String sex;
    /**
     * 电话
     */
    private String phone;
    /**
     * 部门id
     */
    private String deptId;
    /**
     * 邮箱
     */
    private String email;
    /**
     * 创建人
     */
    private String createBy;
    /**
     * 创建日期
     */
    private Date createDate;
    /**
     * 修改人
     */
    private String updateBy;
    /**
     * 修改日期
     */
    private Date updateDate;
    /**
     * 删除标识
     */
    private String delFlag;
    /**
     * 启用标识
     */
    private String flag;
    /**
     * 备注
     */
    private String remarks;

    @TableField(exist = false)
    private Collection<? extends GrantedAuthority> authorities;

    public User(String loginName, String password, String authorities) {
        this.loginName = loginName;
        this.password = password;
        this.authorities = Collections.singleton(new SimpleGrantedAuthority(authorities));
    }


    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return authorities;
    }

    @Override
    public String getUsername() {
        return loginName;
    }


    /**
     * 账户是否过期
     */
    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    /**
     * 账户是否被冻结
     */
    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    /**
     * 账户密码是否过期
     */
    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    /**
     * 账户是否可用
     */
    @Override
    public boolean isEnabled() {
        return true;
    }
}
