package com.isoft91.gonn.gps.sys.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.isoft91.gonn.gps.config.exception.PmsTemplateExecution;
import com.isoft91.gonn.gps.sys.entity.*;
import com.isoft91.gonn.gps.utils.Result.Ret;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 基础信息数据调用
 *
 * @author :jiangpengpeng
 * @date:2020/3/25 13:48
 */
@Service
public class CommonMesServiceImpl {

    @Value("${global.path.user_synchronized}")
    private String USER_SYNCHRONIZED;

    @Value("${global.path.app_id}")
    private String APP_ID;

    @Value("${global.path.regions}")
    private String regions;

    @Value("${global.path.business}")
    private String business;

    @Value("${global.path.operators}")
    private String operators;

    @Value("${global.path.source}")
    private String source;

    @Value("${global.path.customers}")
    private String customers;

    @Value("${global.path.basestate}")
    private String basestate;

    @Value("${global.path.permission}")
    private String permission;

    @Value("${global.path.app_secret}")
    private String app_secret;

    @Value("${global.path.serverAuthenticateStaticKey}")
    private String serverAuthenticateStaticKey;


    //获取token
    public String getToken(String timestamp, String sign) {
        String token = "";
        // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        //创建get请求
        HttpGet httpGet = new HttpGet(USER_SYNCHRONIZED + "/token?app_id=" + APP_ID + "&timestamp=" + timestamp + "&sign=" + sign);
        // 响应模型
        CloseableHttpResponse response = null;
        try {
            // 配置信息
            RequestConfig requestConfig = RequestConfig.custom()
                    // 设置连接超时时间(单位毫秒)
                    .setConnectTimeout(5000)
                    // 设置请求超时时间(单位毫秒)
                    .setConnectionRequestTimeout(5000)
                    // socket读写超时时间(单位毫秒)
                    .setSocketTimeout(5000)
                    // 设置是否允许重定向(默认为true)
                    .setRedirectsEnabled(true).build();
            // 将上面的配置信息 运用到这个Get请求里
            httpGet.setConfig(requestConfig);
            // 由客户端执行(发送)Get请求
            response = httpClient.execute(httpGet);
            // 从响应模型中获取响应实体
            HttpEntity responseEntity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == 200 && responseEntity != null) {
                //取出实体中的数据
                String str = EntityUtils.toString(responseEntity);
                //将数据转换成json数据
                //JSONObject jsonObject = JSONObject.parseObject(str);
                String str1 = JSONObject.parseObject(str).get("data").toString();
                Map mapTypes = JSON.parseObject(str1);
                token = mapTypes.get("token").toString();
            } else {
                return "";
            }
        } catch (Exception e) {
            throw new PmsTemplateExecution("操作失败");
        } finally {
            try {
                // 释放资源
                if (httpClient != null) {
                    httpClient.close();
                }
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return token;
    }


    public List getResponse(String token, Class t, String path, String userId) {
        List tList = new ArrayList<>();
        // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();

        //创建get请求
        HttpGet httpGet = new HttpGet(USER_SYNCHRONIZED + path + "?token=" + token);
        if (!StringUtils.isAnyEmpty(userId)) {
            httpGet = new HttpGet(USER_SYNCHRONIZED + path + "?uid=" + userId + "&token=" + token);

        }
        // 响应模型
        CloseableHttpResponse response = null;

        try {

            // 配置信息
            RequestConfig requestConfig = RequestConfig.custom()
                    // 设置连接超时时间(单位毫秒)
                    .setConnectTimeout(5000)
                    // 设置请求超时时间(单位毫秒)
                    .setConnectionRequestTimeout(5000)
                    // socket读写超时时间(单位毫秒)
                    .setSocketTimeout(5000)
                    // 设置是否允许重定向(默认为true)
                    .setRedirectsEnabled(true).build();

            // 将上面的配置信息 运用到这个Get请求里
            httpGet.setConfig(requestConfig);

            // 由客户端执行(发送)Get请求
            response = httpClient.execute(httpGet);

            // 从响应模型中获取响应实体
            HttpEntity responseEntity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == 200 && responseEntity != null) {
                //取出实体中的数据
                String str = EntityUtils.toString(responseEntity);
                //将数据转换成json数据
                JSONObject jsonObject = JSONObject.parseObject(str);
                tList = JSONObject.parseArray(jsonObject.get("data").toString(), t);

            }
        } catch (Exception e) {

        } finally {
            try {
                // 释放资源
                if (httpClient != null) {
                    httpClient.close();
                }
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return tList;

    }


    public String getPerResponse(String token, Class t, String path, String userId, String flag) {
        String dept = "";
        // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        
        //创建get请求
        HttpGet httpGet = new HttpGet(USER_SYNCHRONIZED + path + "?token=" + token);
        if (!StringUtils.isAnyEmpty(userId)) {
            httpGet = new HttpGet(USER_SYNCHRONIZED + path + "?uid=" + userId + "&token=" + token);

        }
        // 响应模型
        CloseableHttpResponse response = null;

        try {

            // 配置信息
            RequestConfig requestConfig = RequestConfig.custom()
                    // 设置连接超时时间(单位毫秒)
                    .setConnectTimeout(5000)
                    // 设置请求超时时间(单位毫秒)
                    .setConnectionRequestTimeout(5000)
                    // socket读写超时时间(单位毫秒)
                    .setSocketTimeout(5000)
                    // 设置是否允许重定向(默认为true)
                    .setRedirectsEnabled(true).build();

            // 将上面的配置信息 运用到这个Get请求里
            httpGet.setConfig(requestConfig);

            // 由客户端执行(发送)Get请求
            response = httpClient.execute(httpGet);

            // 从响应模型中获取响应实体
            HttpEntity responseEntity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == 200 && responseEntity != null) {
                //取出实体中的数据
                String str = EntityUtils.toString(responseEntity);
                //将数据转换成json数据
                JSONObject jsonObject = JSONObject.parseObject(str);
                JSONArray jsonArray = JSONObject.parseArray(jsonObject.get("data").toString());
                for (int i = 0; i < jsonArray.size(); i++) {
                    JSONArray jsonArray1 = JSONObject.parseArray(jsonArray.get(i).toString());
                    for (int j = 0; j < jsonArray1.size(); j++) {
                        if (jsonArray1.get(j).toString().equals(flag)) {
                            for (int g = 0; g < jsonArray1.size(); g++) {
                                if (!jsonArray1.get(g).toString().equals("in") && !jsonArray1.get(g).toString().equals(flag)) {
                                    dept = jsonArray1.get(g).toString();
                                    break;
                                }
                            }
                        }
                        continue;
                    }
                }

            }
        } catch (Exception e) {

        } finally {
            try {
                // 释放资源
                if (httpClient != null) {
                    httpClient.close();
                }
                if (response != null) {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return dept;

    }

    /**
     * 获取区域
     * [timestamp, sign]
     *
     * @return {@link Ret}
     * @throws
     * @author jiangpengpeng
     * @date 2020/3/25 15:45
     */
    public Ret getArea(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, Area.class, regions, "");
        return Ret.ok().setData(response);
    }

    /**
     * 业务线
     * [timestamp, sign]
     *
     * @return {@link Ret}
     * @throws
     * @author jiangpengpeng
     * @date 2020/3/25 16:05
     */
    public Ret getServiceLine(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, ServiceLine.class, business, "");

        return Ret.ok().setData(response);
    }

    public Ret getOperator(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, Operator.class, operators, "");

        return Ret.ok().setData(response);
    }

    public Ret getNeedSouce(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, NeedSource.class, source, "");

        return Ret.ok().setData(response);
    }

    public Ret getUpper(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, Upper.class, customers, "");

        return Ret.ok().setData(response);
    }

    public Ret getJzState(String timestamp, String sign) {
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        List response = this.getResponse(token, JzState.class, basestate, "");

        return Ret.ok().setData(response);
    }

    public Ret getPermission(String timestamp, String sign, String userId, String flag) throws NoSuchAlgorithmException {
        timestamp = Long.toString(System.currentTimeMillis() / 1000L);
        //timestamp = "1565169731";
        sign = app_secret + timestamp + serverAuthenticateStaticKey;
        sign = CommonMesServiceImpl.sha1(sign);
        sign = DigestUtils.md5DigestAsHex(sign.getBytes());
        String token = this.getToken(timestamp, sign);
        if (StringUtils.isAnyEmpty(token)) {
            return Ret.error().setMsg("token获取失败");
        }

        String response = this.getPerResponse(token, JzState.class, permission, userId, flag);
        String[] split = response.split(",");
        List<String> strings = Arrays.asList(split);
        return Ret.ok().setData(strings);
    }

    /**
     * sha1加密
     *
     * @param
     * @return
     * @throws NoSuchAlgorithmException
     */
    public static String sha1(String decript) {
        try {
            MessageDigest digest = java.security.MessageDigest
                    .getInstance("SHA-1");
            digest.update(decript.getBytes());
            byte messageDigest[] = digest.digest();
            // Create Hex String
            StringBuffer hexString = new StringBuffer();
            // 字节数组转换为 十六进制 数
            for (int i = 0; i < messageDigest.length; i++) {
                String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
                if (shaHex.length() < 2) {
                    hexString.append(0);
                }
                hexString.append(shaHex);
            }
            return hexString.toString();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return "";
    }

}
